@@ -369,7 +369,7 @@ func NewValidationFromTableStruct(v *table.Validation) *RPKIValidation {
 	}
 }
 
-func ToPathApi(path *table.Path) *Path {
+func ToPathApi(path *table.Path, v *table.Validation) *Path {
 	nlri := path.GetNlri()
 	n, _ := nlri.Serialize()
 	family := uint32(bgp.AfiSafiToRouteFamily(nlri.AFI(), nlri.SAFI()))
@@ -381,19 +381,22 @@ func ToPathApi(path *table.Path) *Path {
 		}
 		return ret
 	}(path.GetPathAttrs())
+	vv := config.RPKI_VALIDATION_RESULT_TYPE_NONE.ToInt()
+	if v != nil {
+		vv = v.Status.ToInt()
+	}
+
 	p := &Path{
 		Nlri:               n,
 		Pattrs:             pattrs,
 		Age:                path.GetTimestamp().Unix(),
 		IsWithdraw:         path.IsWithdraw,
-		Validation:         int32(path.ValidationStatus().ToInt()),
-		ValidationDetail:   NewValidationFromTableStruct(path.Validation()),
-		Filtered:           path.Filtered("") == table.POLICY_DIRECTION_IN,
+		Validation:         int32(vv),
+		ValidationDetail:   NewValidationFromTableStruct(v),
 		Family:             family,
 		Stale:              path.IsStale(),
 		IsFromExternal:     path.IsFromExternal(),
 		NoImplicitWithdraw: path.NoImplicitWithdraw(),
-		Uuid:               path.UUID().Bytes(),
 		IsNexthopInvalid:   path.IsNexthopInvalid,
 		Identifier:         nlri.PathIdentifier(),
 		LocalIdentifier:    nlri.PathLocalIdentifier(),
@@ -406,6 +409,14 @@ func ToPathApi(path *table.Path) *Path {
 	return p
 }
 
+func getValidation(v []*table.Validation, i int) *table.Validation {
+	if v == nil {
+		return nil
+	} else {
+		return v[i]
+	}
+}
+
 func (s *Server) GetRib(ctx context.Context, arg *GetRibRequest) (*GetRibResponse, error) {
 	if arg == nil || arg.Table == nil {
 		return nil, fmt.Errorf("invalid request")
@@ -431,16 +442,17 @@ func (s *Server) GetRib(ctx context.Context, arg *GetRibRequest) (*GetRibRespons
 	var in bool
 	var err error
 	var tbl *table.Table
+	var v []*table.Validation
 
 	family := bgp.RouteFamily(arg.Table.Family)
 	switch arg.Table.Type {
 	case Resource_LOCAL, Resource_GLOBAL:
-		tbl, err = s.bgpServer.GetRib(arg.Table.Name, family, f())
+		tbl, v, err = s.bgpServer.GetRib(arg.Table.Name, family, f())
 	case Resource_ADJ_IN:
 		in = true
 		fallthrough
 	case Resource_ADJ_OUT:
-		tbl, err = s.bgpServer.GetAdjRib(arg.Table.Name, family, in, f())
+		tbl, v, err = s.bgpServer.GetAdjRib(arg.Table.Name, family, in, f())
 	case Resource_VRF:
 		tbl, err = s.bgpServer.GetVrfRib(arg.Table.Name, family, []*table.LookupPrefix{})
 	default:
@@ -453,13 +465,15 @@ func (s *Server) GetRib(ctx context.Context, arg *GetRibRequest) (*GetRibRespons
 
 	tblDsts := tbl.GetDestinations()
 	dsts := make([]*Destination, 0, len(tblDsts))
+	idx := 0
 	for _, dst := range tblDsts {
 		dsts = append(dsts, &Destination{
 			Prefix: dst.GetNlri().String(),
 			Paths: func(paths []*table.Path) []*Path {
 				l := make([]*Path, 0, len(paths))
 				for i, p := range paths {
-					pp := ToPathApi(p)
+					pp := ToPathApi(p, getValidation(v, idx))
+					idx++
 					switch arg.Table.Type {
 					case Resource_LOCAL, Resource_GLOBAL:
 						if i == 0 && !table.SelectionOptions.DisableBestPathSelection {
@@ -496,14 +510,15 @@ func (s *Server) GetPath(arg *GetPathRequest, stream GobgpApi_GetPathServer) err
 	family := bgp.RouteFamily(arg.Family)
 	var tbl *table.Table
 	var err error
+	var v []*table.Validation
 	switch arg.Type {
 	case Resource_LOCAL, Resource_GLOBAL:
-		tbl, err = s.bgpServer.GetRib(arg.Name, family, f())
+		tbl, v, err = s.bgpServer.GetRib(arg.Name, family, f())
 	case Resource_ADJ_IN:
 		in = true
 		fallthrough
 	case Resource_ADJ_OUT:
-		tbl, err = s.bgpServer.GetAdjRib(arg.Name, family, in, f())
+		tbl, v, err = s.bgpServer.GetAdjRib(arg.Name, family, in, f())
 	case Resource_VRF:
 		tbl, err = s.bgpServer.GetVrfRib(arg.Name, family, []*table.LookupPrefix{})
 	default:
@@ -513,11 +528,13 @@ func (s *Server) GetPath(arg *GetPathRequest, stream GobgpApi_GetPathServer) err
 		return err
 	}
 
+	idx := 0
 	return func() error {
 		for _, dst := range tbl.GetDestinations() {
-			for idx, path := range dst.GetAllKnownPathList() {
-				p := ToPathApi(path)
-				if idx == 0 && !table.SelectionOptions.DisableBestPathSelection {
+			for i, path := range dst.GetAllKnownPathList() {
+				p := ToPathApi(path, getValidation(v, idx))
+				idx++
+				if i == 0 && !table.SelectionOptions.DisableBestPathSelection {
 					switch arg.Type {
 					case Resource_LOCAL, Resource_GLOBAL:
 						p.Best = true
@@ -564,11 +581,11 @@ func (s *Server) MonitorRib(arg *MonitorRibRequest, stream GobgpApi_MonitorRibSe
 					continue
 				}
 				if dst, y := dsts[path.GetNlri().String()]; y {
-					dst.Paths = append(dst.Paths, ToPathApi(path))
+					dst.Paths = append(dst.Paths, ToPathApi(path, nil))
 				} else {
 					dsts[path.GetNlri().String()] = &Destination{
 						Prefix: path.GetNlri().String(),
-						Paths:  []*Path{ToPathApi(path)},
+						Paths:  []*Path{ToPathApi(path, nil)},
 					}
 				}
 			}
@@ -865,7 +882,7 @@ func (s *Server) DeleteBmp(ctx context.Context, arg *DeleteBmpRequest) (*DeleteB
 }
 
 func (s *Server) ValidateRib(ctx context.Context, arg *ValidateRibRequest) (*ValidateRibResponse, error) {
-	return &ValidateRibResponse{}, s.bgpServer.ValidateRib(arg.Prefix)
+	return &ValidateRibResponse{}, nil
 }
 
 func (s *Server) AddRpki(ctx context.Context, arg *AddRpkiRequest) (*AddRpkiResponse, error) {
@@ -1343,6 +1360,21 @@ func (s *Server) DeleteNeighbor(ctx context.Context, arg *DeleteNeighborRequest)
 	}})
 }
 
+func (s *Server) UpdateNeighbor(ctx context.Context, arg *UpdateNeighborRequest) (*UpdateNeighborResponse, error) {
+	c, err := NewNeighborFromAPIStruct(arg.Peer)
+	if err != nil {
+		return nil, err
+	}
+	needsSoftResetIn, err := s.bgpServer.UpdateNeighbor(c)
+	if err != nil {
+		return nil, err
+	}
+	if arg.DoSoftResetIn && needsSoftResetIn {
+		return &UpdateNeighborResponse{NeedsSoftResetIn: false}, s.bgpServer.SoftResetIn("", bgp.RouteFamily(0))
+	}
+	return &UpdateNeighborResponse{NeedsSoftResetIn: needsSoftResetIn}, nil
+}
+
 func NewPrefixFromApiStruct(a *Prefix) (*table.Prefix, error) {
 	_, prefix, err := net.ParseCIDR(a.IpPrefix)
 	if err != nil {
@@ -2132,8 +2164,6 @@ func NewAPIPolicyAssignmentFromTableStruct(t *table.PolicyAssignment) *PolicyAss
 	return &PolicyAssignment{
 		Type: func() PolicyType {
 			switch t.Type {
-			case table.POLICY_DIRECTION_IN:
-				return PolicyType_IN
 			case table.POLICY_DIRECTION_IMPORT:
 				return PolicyType_IMPORT
 			case table.POLICY_DIRECTION_EXPORT:
@@ -2261,8 +2291,6 @@ func toPolicyAssignmentName(a *PolicyAssignment) (string, table.PolicyDirection,
 		}
 	case Resource_LOCAL:
 		switch a.Type {
-		case PolicyType_IN:
-			return a.Name, table.POLICY_DIRECTION_IN, nil
 		case PolicyType_IMPORT:
 			return a.Name, table.POLICY_DIRECTION_IMPORT, nil
 		case PolicyType_EXPORT:
