@@ -565,8 +565,8 @@ func checkOriginAsWasNotShown(p *table.Path, shownAs map[uint32]struct{}) bool {
 	if len(asPath) == 0 {
 		return false
 	}
-	aslist := asPath[len(asPath)-1].(*bgp.As4PathParam).AS
-	origin := aslist[len(aslist)-1]
+	asList := asPath[len(asPath)-1].GetAS()
+	origin := asList[len(asList)-1]
 
 	if _, ok := shownAs[origin]; ok {
 		return false
@@ -585,8 +585,8 @@ func showValidationInfo(p *table.Path, shownAs map[uint32]struct{}) error {
 
 	status := p.Validation().Status
 	reason := p.Validation().Reason
-	aslist := asPath[len(asPath)-1].(*bgp.As4PathParam).AS
-	origin := aslist[len(aslist)-1]
+	asList := asPath[len(asPath)-1].GetAS()
+	origin := asList[len(asList)-1]
 
 	fmt.Printf("Target Prefix: %s, AS: %d\n", p.GetNlri().String(), origin)
 	fmt.Printf("  This route is %s", status)
@@ -701,6 +701,7 @@ func showNeighborRib(r string, name string, args []string) error {
 		showAge = false
 	case CMD_VRF:
 		def = bgp.RF_IPv4_UC
+		showBest = true
 	}
 	family, err := checkAddressFamily(def)
 	if err != nil {
@@ -768,7 +769,7 @@ func showNeighborRib(r string, name string, args []string) error {
 
 	switch r {
 	case CMD_LOCAL, CMD_ADJ_IN, CMD_ACCEPTED, CMD_REJECTED, CMD_ADJ_OUT:
-		if rib.Info("").NumDestination == 0 {
+		if rib.Info("", 0).NumDestination == 0 {
 			peer, err := client.GetNeighbor(name, false)
 			if err != nil {
 				return err
@@ -780,14 +781,28 @@ func showNeighborRib(r string, name string, args []string) error {
 	}
 
 	if globalOpts.Json {
-		j, _ := json.Marshal(rib.GetDestinations())
+		d := make(map[string]*table.Destination)
+		for _, dst := range rib.GetDestinations() {
+			d[dst.GetNlri().String()] = dst
+		}
+		j, _ := json.Marshal(d)
 		fmt.Println(string(j))
 		return nil
 	}
 
 	if validationTarget != "" {
 		// show RPKI validation info
-		d := rib.GetDestination(validationTarget)
+		addr, _, err := net.ParseCIDR(validationTarget)
+		if err != nil {
+			return err
+		}
+		var nlri bgp.AddrPrefixInterface
+		if addr.To16() == nil {
+			nlri, _ = bgp.NewPrefixFromRouteFamily(bgp.AFI_IP, bgp.SAFI_UNICAST, validationTarget)
+		} else {
+			nlri, _ = bgp.NewPrefixFromRouteFamily(bgp.AFI_IP6, bgp.SAFI_UNICAST, validationTarget)
+		}
+		d := rib.GetDestination(nlri)
 		if d == nil {
 			fmt.Println("Network not in table")
 			return nil
@@ -806,18 +821,10 @@ func showNeighborRib(r string, name string, args []string) error {
 			switch r {
 			case CMD_ACCEPTED:
 				for _, p := range d.GetAllKnownPathList() {
-					if p.Filtered("") > table.POLICY_DIRECTION_NONE {
-						continue
-					}
 					ps = append(ps, p)
 				}
 			case CMD_REJECTED:
-				for _, p := range d.GetAllKnownPathList() {
-					if p.Filtered("") == table.POLICY_DIRECTION_NONE {
-						continue
-					}
-					ps = append(ps, p)
-				}
+				// always nothing
 			default:
 				ps = d.GetAllKnownPathList()
 			}
@@ -871,8 +878,6 @@ func showNeighborPolicy(remoteIP, policyType string, indent int) error {
 	var err error
 
 	switch strings.ToLower(policyType) {
-	case "in":
-		assignment, err = client.GetRouteServerInPolicy(remoteIP)
 	case "import":
 		assignment, err = client.GetRouteServerImportPolicy(remoteIP)
 	case "export":
@@ -925,8 +930,6 @@ func modNeighborPolicy(remoteIP, policyType, cmdType string, args []string) erro
 		Name: remoteIP,
 	}
 	switch strings.ToLower(policyType) {
-	case "in":
-		assign.Type = table.POLICY_DIRECTION_IN
 	case "import":
 		assign.Type = table.POLICY_DIRECTION_IMPORT
 	case "export":
@@ -973,9 +976,16 @@ func modNeighborPolicy(remoteIP, policyType, cmdType string, args []string) erro
 }
 
 func modNeighbor(cmdType string, args []string) error {
-	params := map[string]int{"interface": PARAM_SINGLE}
+	params := map[string]int{
+		"interface": PARAM_SINGLE,
+	}
 	usage := fmt.Sprintf("usage: gobgp neighbor %s [ <neighbor-address> | interface <neighbor-interface> ]", cmdType)
 	if cmdType == CMD_ADD {
+		usage += " as <VALUE>"
+	} else if cmdType == CMD_UPDATE {
+		usage += " [ as <VALUE> ]"
+	}
+	if cmdType == CMD_ADD || cmdType == CMD_UPDATE {
 		params["as"] = PARAM_SINGLE
 		params["family"] = PARAM_SINGLE
 		params["vrf"] = PARAM_SINGLE
@@ -984,13 +994,14 @@ func modNeighbor(cmdType string, args []string) error {
 		params["allow-own-as"] = PARAM_SINGLE
 		params["remove-private-as"] = PARAM_SINGLE
 		params["replace-peer-as"] = PARAM_FLAG
-		usage += " as <VALUE> [ family <address-families-list> | vrf <vrf-name> | route-reflector-client [<cluster-id>] | route-server-client | allow-own-as <num> | remove-private-as (all|replace) | replace-peer-as ]"
+		usage += " [ family <address-families-list> | vrf <vrf-name> | route-reflector-client [<cluster-id>] | route-server-client | allow-own-as <num> | remove-private-as (all|replace) | replace-peer-as ]"
 	}
 
 	m, err := extractReserved(args, params)
 	if err != nil || (len(m[""]) != 1 && len(m["interface"]) != 1) {
 		return fmt.Errorf("%s", usage)
 	}
+
 	unnumbered := len(m["interface"]) > 0
 	if !unnumbered {
 		if _, err := net.ResolveIPAddr("ip", m[""][0]); err != nil {
@@ -998,28 +1009,53 @@ func modNeighbor(cmdType string, args []string) error {
 		}
 	}
 
-	getConf := func(asn uint32) (*config.Neighbor, error) {
-		peer := &config.Neighbor{
-			Config: config.NeighborConfig{
-				PeerAs: asn,
-			},
-		}
+	getNeighborAddress := func() (string, error) {
 		if unnumbered {
-			peer.Config.NeighborInterface = m["interface"][0]
-			addr, err := config.GetIPv6LinkLocalNeighborAddress(peer.Config.NeighborInterface)
+			return config.GetIPv6LinkLocalNeighborAddress(m["interface"][0])
+		}
+		return m[""][0], nil
+	}
+
+	getNeighborConfig := func() (*config.Neighbor, error) {
+		addr, err := getNeighborAddress()
+		if err != nil {
+			return nil, err
+		}
+		var peer *config.Neighbor
+		switch cmdType {
+		case CMD_ADD, CMD_DEL:
+			peer = &config.Neighbor{}
+			if unnumbered {
+				peer.Config.NeighborInterface = m["interface"][0]
+			} else {
+				peer.Config.NeighborAddress = addr
+			}
+			peer.State.NeighborAddress = addr
+		case CMD_UPDATE:
+			peer, err = client.GetNeighbor(addr)
 			if err != nil {
 				return nil, err
 			}
-			peer.State.NeighborAddress = addr
-		} else {
-			peer.Config.NeighborAddress = m[""][0]
-			peer.State.NeighborAddress = m[""][0]
+		default:
+			return nil, fmt.Errorf("invalid command: %s", cmdType)
+		}
+		return peer, nil
+	}
+
+	updateNeighborConfig := func(peer *config.Neighbor) error {
+		if len(m["as"]) > 0 {
+			as, err := strconv.ParseUint(m["as"][0], 10, 32)
+			if err != nil {
+				return err
+			}
+			peer.Config.PeerAs = uint32(as)
 		}
 		if len(m["family"]) == 1 {
+			peer.AfiSafis = make([]config.AfiSafi, 0) // for the case of CMD_UPDATE
 			for _, family := range strings.Split(m["family"][0], ",") {
 				afiSafiName := config.AfiSafiType(family)
 				if afiSafiName.ToInt() == -1 {
-					return nil, fmt.Errorf("invalid family value: %s", family)
+					return fmt.Errorf("invalid family value: %s", family)
 				}
 				peer.AfiSafis = append(peer.AfiSafis, config.AfiSafi{Config: config.AfiSafiConfig{AfiSafiName: afiSafiName}})
 			}
@@ -1027,12 +1063,12 @@ func modNeighbor(cmdType string, args []string) error {
 		if len(m["vrf"]) == 1 {
 			peer.Config.Vrf = m["vrf"][0]
 		}
-		if rr, ok := m["route-reflector-client"]; ok {
+		if option, ok := m["route-reflector-client"]; ok {
 			peer.RouteReflector.Config = config.RouteReflectorConfig{
 				RouteReflectorClient: true,
 			}
-			if len(rr) == 1 {
-				peer.RouteReflector.Config.RouteReflectorClusterId = config.RrClusterIdType(rr[0])
+			if len(option) == 1 {
+				peer.RouteReflector.Config.RouteReflectorClusterId = config.RrClusterIdType(option[0])
 			}
 		}
 		if _, ok := m["route-server-client"]; ok {
@@ -1043,7 +1079,7 @@ func modNeighbor(cmdType string, args []string) error {
 		if option, ok := m["allow-own-as"]; ok {
 			as, err := strconv.ParseUint(option[0], 10, 8)
 			if err != nil {
-				return nil, err
+				return err
 			}
 			peer.AsPathOptions.Config.AllowOwnAs = uint8(as)
 		}
@@ -1054,36 +1090,34 @@ func modNeighbor(cmdType string, args []string) error {
 			case "replace":
 				peer.Config.RemovePrivateAs = config.REMOVE_PRIVATE_AS_OPTION_REPLACE
 			default:
-				return nil, fmt.Errorf("invalid remove-private-as value: all or replace")
+				return fmt.Errorf("invalid remove-private-as value: all or replace")
 			}
 		}
 		if _, ok := m["replace-peer-as"]; ok {
 			peer.AsPathOptions.Config.ReplacePeerAs = true
 		}
-		return peer, nil
+		return nil
 	}
 
-	var as uint64
-	if len(m["as"]) > 0 {
-		var err error
-		if as, err = strconv.ParseUint(m["as"][0], 10, 32); err != nil {
-			return err
-		}
-	}
-
-	n, err := getConf(uint32(as))
+	n, err := getNeighborConfig()
 	if err != nil {
 		return err
 	}
 
 	switch cmdType {
 	case CMD_ADD:
-		if len(m[""]) > 0 && len(m["as"]) != 1 {
-			return fmt.Errorf("%s", usage)
+		if err := updateNeighborConfig(n); err != nil {
+			return err
 		}
 		return client.AddNeighbor(n)
 	case CMD_DEL:
 		return client.DeleteNeighbor(n)
+	case CMD_UPDATE:
+		if err := updateNeighborConfig(n); err != nil {
+			return err
+		}
+		_, err := client.UpdateNeighbor(n, true)
+		return err
 	}
 	return nil
 }
@@ -1219,7 +1253,7 @@ func NewNeighborCmd() *cobra.Command {
 		},
 	}
 
-	for _, v := range []string{CMD_ADD, CMD_DEL} {
+	for _, v := range []string{CMD_ADD, CMD_DEL, CMD_UPDATE} {
 		cmd := &cobra.Command{
 			Use: v,
 			Run: func(c *cobra.Command, args []string) {
